"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from flexprice_py import errors, models, utils
from flexprice_py._hooks import HookContext
from flexprice_py.types import OptionalNullable, UNSET
from flexprice_py.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union


class Costs(BaseSDK):
    def create_costsheet(
        self,
        *,
        name: str,
        description: Optional[str] = None,
        lookup_key: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCreateCostsheetResponse:
        r"""Create costsheet

        Use when setting up a new pricing configuration (e.g. a new product or region). Costsheets group prices and define the default for the environment.

        :param name:
        :param description:
        :param lookup_key:
        :param metadata:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreateCostsheetRequest(
            description=description,
            lookup_key=lookup_key,
            metadata=metadata,
            name=name,
        )

        req = self._build_request(
            method="POST",
            path="/costs",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.components.DtoCreateCostsheetRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createCostsheet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "409", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCreateCostsheetResponse, http_res
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def create_costsheet_async(
        self,
        *,
        name: str,
        description: Optional[str] = None,
        lookup_key: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCreateCostsheetResponse:
        r"""Create costsheet

        Use when setting up a new pricing configuration (e.g. a new product or region). Costsheets group prices and define the default for the environment.

        :param name:
        :param description:
        :param lookup_key:
        :param metadata:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreateCostsheetRequest(
            description=description,
            lookup_key=lookup_key,
            metadata=metadata,
            name=name,
        )

        req = self._build_request_async(
            method="POST",
            path="/costs",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.components.DtoCreateCostsheetRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createCostsheet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "409", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCreateCostsheetResponse, http_res
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_active_costsheet(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCostsheetResponse:
        r"""Get active costsheet

        Use when you need the tenant's default pricing configuration (e.g. for checkout or plan display). Returns the active costsheet for the environment.

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request(
            method="GET",
            path="/costs/active",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getActiveCostsheet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCostsheetResponse, http_res
            )
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_active_costsheet_async(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCostsheetResponse:
        r"""Get active costsheet

        Use when you need the tenant's default pricing configuration (e.g. for checkout or plan display). Returns the active costsheet for the environment.

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request_async(
            method="GET",
            path="/costs/active",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getActiveCostsheet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCostsheetResponse, http_res
            )
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_detailed_cost_analytics(
        self,
        *,
        end_time: Optional[str] = None,
        expand: Optional[List[str]] = None,
        external_customer_id: Optional[str] = None,
        feature_ids: Optional[List[str]] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        start_time: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoGetDetailedCostAnalyticsResponse:
        r"""Get combined revenue and cost analytics

        Use when building dashboards or reports that need revenue vs cost, ROI, and margin over a time period (e.g. finance views or executive summaries).

        :param end_time:
        :param expand: Expand options - specify which entities to expand
        :param external_customer_id: Optional - for specific customer
        :param feature_ids: Additional filters
        :param limit: Pagination
        :param offset:
        :param start_time: Time range fields (optional - defaults to last 7 days if not provided)
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoGetCostAnalyticsRequest(
            end_time=end_time,
            expand=expand,
            external_customer_id=external_customer_id,
            feature_ids=feature_ids,
            limit=limit,
            offset=offset,
            start_time=start_time,
        )

        req = self._build_request(
            method="POST",
            path="/costs/analytics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.components.DtoGetCostAnalyticsRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getDetailedCostAnalytics",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoGetDetailedCostAnalyticsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_detailed_cost_analytics_async(
        self,
        *,
        end_time: Optional[str] = None,
        expand: Optional[List[str]] = None,
        external_customer_id: Optional[str] = None,
        feature_ids: Optional[List[str]] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        start_time: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoGetDetailedCostAnalyticsResponse:
        r"""Get combined revenue and cost analytics

        Use when building dashboards or reports that need revenue vs cost, ROI, and margin over a time period (e.g. finance views or executive summaries).

        :param end_time:
        :param expand: Expand options - specify which entities to expand
        :param external_customer_id: Optional - for specific customer
        :param feature_ids: Additional filters
        :param limit: Pagination
        :param offset:
        :param start_time: Time range fields (optional - defaults to last 7 days if not provided)
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoGetCostAnalyticsRequest(
            end_time=end_time,
            expand=expand,
            external_customer_id=external_customer_id,
            feature_ids=feature_ids,
            limit=limit,
            offset=offset,
            start_time=start_time,
        )

        req = self._build_request_async(
            method="POST",
            path="/costs/analytics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.components.DtoGetCostAnalyticsRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getDetailedCostAnalytics",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoGetDetailedCostAnalyticsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_detailed_cost_analytics_v2(
        self,
        *,
        end_time: Optional[str] = None,
        expand: Optional[List[str]] = None,
        external_customer_id: Optional[str] = None,
        feature_ids: Optional[List[str]] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        start_time: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoGetDetailedCostAnalyticsResponse:
        r"""Get combined revenue and cost analytics (V2)

        Use when you need the same revenue/cost/ROI analytics but computed from the costsheet usage-tracking pipeline (e.g. for consistency with usage-based cost data).

        :param end_time:
        :param expand: Expand options - specify which entities to expand
        :param external_customer_id: Optional - for specific customer
        :param feature_ids: Additional filters
        :param limit: Pagination
        :param offset:
        :param start_time: Time range fields (optional - defaults to last 7 days if not provided)
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoGetCostAnalyticsRequest(
            end_time=end_time,
            expand=expand,
            external_customer_id=external_customer_id,
            feature_ids=feature_ids,
            limit=limit,
            offset=offset,
            start_time=start_time,
        )

        req = self._build_request(
            method="POST",
            path="/costs/analytics-v2",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.components.DtoGetCostAnalyticsRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getDetailedCostAnalyticsV2",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoGetDetailedCostAnalyticsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_detailed_cost_analytics_v2_async(
        self,
        *,
        end_time: Optional[str] = None,
        expand: Optional[List[str]] = None,
        external_customer_id: Optional[str] = None,
        feature_ids: Optional[List[str]] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        start_time: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoGetDetailedCostAnalyticsResponse:
        r"""Get combined revenue and cost analytics (V2)

        Use when you need the same revenue/cost/ROI analytics but computed from the costsheet usage-tracking pipeline (e.g. for consistency with usage-based cost data).

        :param end_time:
        :param expand: Expand options - specify which entities to expand
        :param external_customer_id: Optional - for specific customer
        :param feature_ids: Additional filters
        :param limit: Pagination
        :param offset:
        :param start_time: Time range fields (optional - defaults to last 7 days if not provided)
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoGetCostAnalyticsRequest(
            end_time=end_time,
            expand=expand,
            external_customer_id=external_customer_id,
            feature_ids=feature_ids,
            limit=limit,
            offset=offset,
            start_time=start_time,
        )

        req = self._build_request_async(
            method="POST",
            path="/costs/analytics-v2",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.components.DtoGetCostAnalyticsRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getDetailedCostAnalyticsV2",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoGetDetailedCostAnalyticsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def query_costsheet(
        self,
        *,
        costsheet_i_ds: Optional[List[str]] = None,
        environment_id: Optional[str] = None,
        filters: Optional[
            Union[
                List[models.components.FilterCondition],
                List[models.components.FilterConditionTypedDict],
            ]
        ] = None,
        lookup_key: Optional[str] = None,
        name: Optional[str] = None,
        query_filter: Optional[
            Union[models.components.QueryFilter, models.components.QueryFilterTypedDict]
        ] = None,
        sort: Optional[
            Union[
                List[models.components.SortCondition],
                List[models.components.SortConditionTypedDict],
            ]
        ] = None,
        status: Optional[models.components.Status] = None,
        tenant_id: Optional[str] = None,
        time_range_filter: Optional[
            Union[
                models.components.TimeRangeFilter,
                models.components.TimeRangeFilterTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListCostsheetResponse:
        r"""Query costsheets

        Use when listing or searching costsheets (e.g. admin catalog). Returns a paginated list; supports filtering and sorting.

        :param costsheet_i_ds: CostsheetIDs allows filtering by specific costsheet IDs
        :param environment_id: EnvironmentID filters by specific environment ID
        :param filters: Filters contains custom filtering conditions
        :param lookup_key: LookupKey filters by lookup key
        :param name: Name filters by costsheet name
        :param query_filter:
        :param sort: Sort specifies result ordering preferences
        :param status:
        :param tenant_id: TenantID filters by specific tenant ID
        :param time_range_filter:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.CostsheetFilter(
            costsheet_i_ds=costsheet_i_ds,
            environment_id=environment_id,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.components.FilterCondition]]
            ),
            lookup_key=lookup_key,
            name=name,
            query_filter=utils.get_pydantic_model(
                query_filter, Optional[models.components.QueryFilter]
            ),
            sort=utils.get_pydantic_model(
                sort, Optional[List[models.components.SortCondition]]
            ),
            status=status,
            tenant_id=tenant_id,
            time_range_filter=utils.get_pydantic_model(
                time_range_filter, Optional[models.components.TimeRangeFilter]
            ),
        )

        req = self._build_request(
            method="POST",
            path="/costs/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.CostsheetFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryCostsheet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListCostsheetResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def query_costsheet_async(
        self,
        *,
        costsheet_i_ds: Optional[List[str]] = None,
        environment_id: Optional[str] = None,
        filters: Optional[
            Union[
                List[models.components.FilterCondition],
                List[models.components.FilterConditionTypedDict],
            ]
        ] = None,
        lookup_key: Optional[str] = None,
        name: Optional[str] = None,
        query_filter: Optional[
            Union[models.components.QueryFilter, models.components.QueryFilterTypedDict]
        ] = None,
        sort: Optional[
            Union[
                List[models.components.SortCondition],
                List[models.components.SortConditionTypedDict],
            ]
        ] = None,
        status: Optional[models.components.Status] = None,
        tenant_id: Optional[str] = None,
        time_range_filter: Optional[
            Union[
                models.components.TimeRangeFilter,
                models.components.TimeRangeFilterTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListCostsheetResponse:
        r"""Query costsheets

        Use when listing or searching costsheets (e.g. admin catalog). Returns a paginated list; supports filtering and sorting.

        :param costsheet_i_ds: CostsheetIDs allows filtering by specific costsheet IDs
        :param environment_id: EnvironmentID filters by specific environment ID
        :param filters: Filters contains custom filtering conditions
        :param lookup_key: LookupKey filters by lookup key
        :param name: Name filters by costsheet name
        :param query_filter:
        :param sort: Sort specifies result ordering preferences
        :param status:
        :param tenant_id: TenantID filters by specific tenant ID
        :param time_range_filter:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.CostsheetFilter(
            costsheet_i_ds=costsheet_i_ds,
            environment_id=environment_id,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.components.FilterCondition]]
            ),
            lookup_key=lookup_key,
            name=name,
            query_filter=utils.get_pydantic_model(
                query_filter, Optional[models.components.QueryFilter]
            ),
            sort=utils.get_pydantic_model(
                sort, Optional[List[models.components.SortCondition]]
            ),
            status=status,
            tenant_id=tenant_id,
            time_range_filter=utils.get_pydantic_model(
                time_range_filter, Optional[models.components.TimeRangeFilter]
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/costs/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.CostsheetFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryCostsheet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListCostsheetResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_costsheet(
        self,
        *,
        id: str,
        expand: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoGetCostsheetResponse:
        r"""Get costsheet

        Use when you need to load a single costsheet (e.g. for editing or display). Supports optional expand for related prices.

        :param id: Costsheet ID
        :param expand: Comma-separated list of fields to expand (e.g., 'prices')
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetCostsheetRequest(
            id=id,
            expand=expand,
        )

        req = self._build_request(
            method="GET",
            path="/costs/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getCostsheet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoGetCostsheetResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_costsheet_async(
        self,
        *,
        id: str,
        expand: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoGetCostsheetResponse:
        r"""Get costsheet

        Use when you need to load a single costsheet (e.g. for editing or display). Supports optional expand for related prices.

        :param id: Costsheet ID
        :param expand: Comma-separated list of fields to expand (e.g., 'prices')
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetCostsheetRequest(
            id=id,
            expand=expand,
        )

        req = self._build_request_async(
            method="GET",
            path="/costs/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getCostsheet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoGetCostsheetResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def update_costsheet(
        self,
        *,
        id: str,
        description: Optional[str] = None,
        lookup_key: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        name: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoUpdateCostsheetResponse:
        r"""Update costsheet

        Use when changing costsheet name or metadata.

        :param id: Costsheet ID
        :param description:
        :param lookup_key:
        :param metadata:
        :param name:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.UpdateCostsheetRequest(
            id=id,
            body=models.components.DtoUpdateCostsheetRequest(
                description=description,
                lookup_key=lookup_key,
                metadata=metadata,
                name=name,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/costs/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoUpdateCostsheetRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateCostsheet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "409", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoUpdateCostsheetResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404", "409"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def update_costsheet_async(
        self,
        *,
        id: str,
        description: Optional[str] = None,
        lookup_key: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        name: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoUpdateCostsheetResponse:
        r"""Update costsheet

        Use when changing costsheet name or metadata.

        :param id: Costsheet ID
        :param description:
        :param lookup_key:
        :param metadata:
        :param name:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.UpdateCostsheetRequest(
            id=id,
            body=models.components.DtoUpdateCostsheetRequest(
                description=description,
                lookup_key=lookup_key,
                metadata=metadata,
                name=name,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/costs/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoUpdateCostsheetRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateCostsheet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "409", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoUpdateCostsheetResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404", "409"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def delete_costsheet(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoDeleteCostsheetResponse:
        r"""Delete costsheet

        Use when retiring a costsheet (e.g. end-of-life product). Soft-deletes; status set to deleted.

        :param id: Costsheet ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.DeleteCostsheetRequest(
            id=id,
        )

        req = self._build_request(
            method="DELETE",
            path="/costs/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deleteCostsheet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoDeleteCostsheetResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def delete_costsheet_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoDeleteCostsheetResponse:
        r"""Delete costsheet

        Use when retiring a costsheet (e.g. end-of-life product). Soft-deletes; status set to deleted.

        :param id: Costsheet ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.DeleteCostsheetRequest(
            id=id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/costs/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deleteCostsheet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoDeleteCostsheetResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import UnrecognizedStr
from typing import Literal, Union


TransactionReason = Union[
    Literal[
        "INVOICE_PAYMENT",
        "FREE_CREDIT_GRANT",
        "SUBSCRIPTION_CREDIT_GRANT",
        "PURCHASED_CREDIT_INVOICED",
        "PURCHASED_CREDIT_DIRECT",
        "CREDIT_NOTE",
        "CREDIT_EXPIRED",
        "WALLET_TERMINATION",
        "MANUAL_BALANCE_DEBIT",
        "CREDIT_ADJUSTMENT",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingmodel import BillingModel
from .billingperiod import BillingPeriod
from .billingtier import BillingTier
from .commitmenttype import CommitmentType
from .dto_createpricetier import DtoCreatePriceTier, DtoCreatePriceTierTypedDict
from .price_transformquantity import (
    PriceTransformQuantity,
    PriceTransformQuantityTypedDict,
)
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoUpdateSubscriptionLineItemRequestTypedDict(TypedDict):
    amount: NotRequired[str]
    r"""Amount is the new price amount that overrides the original price"""
    billing_model: NotRequired[BillingModel]
    commitment_amount: NotRequired[float]
    r"""Commitment fields"""
    commitment_duration: NotRequired[BillingPeriod]
    commitment_overage_factor: NotRequired[float]
    commitment_quantity: NotRequired[float]
    commitment_true_up_enabled: NotRequired[bool]
    commitment_type: NotRequired[CommitmentType]
    commitment_windowed: NotRequired[bool]
    effective_from: NotRequired[str]
    r"""EffectiveFrom for the existing line item (if not provided, defaults to now)"""
    metadata: NotRequired[Dict[str, str]]
    r"""Metadata for the new line item"""
    tier_mode: NotRequired[BillingTier]
    tiers: NotRequired[List[DtoCreatePriceTierTypedDict]]
    r"""Tiers determines the pricing tiers for this line item"""
    transform_quantity: NotRequired[PriceTransformQuantityTypedDict]


class DtoUpdateSubscriptionLineItemRequest(BaseModel):
    amount: Optional[str] = None
    r"""Amount is the new price amount that overrides the original price"""

    billing_model: Optional[BillingModel] = None

    commitment_amount: Optional[float] = None
    r"""Commitment fields"""

    commitment_duration: Optional[BillingPeriod] = None

    commitment_overage_factor: Optional[float] = None

    commitment_quantity: Optional[float] = None

    commitment_true_up_enabled: Optional[bool] = None

    commitment_type: Optional[CommitmentType] = None

    commitment_windowed: Optional[bool] = None

    effective_from: Optional[str] = None
    r"""EffectiveFrom for the existing line item (if not provided, defaults to now)"""

    metadata: Optional[Dict[str, str]] = None
    r"""Metadata for the new line item"""

    tier_mode: Optional[BillingTier] = None

    tiers: Optional[List[DtoCreatePriceTier]] = None
    r"""Tiers determines the pricing tiers for this line item"""

    transform_quantity: Optional[PriceTransformQuantity] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount",
                "billing_model",
                "commitment_amount",
                "commitment_duration",
                "commitment_overage_factor",
                "commitment_quantity",
                "commitment_true_up_enabled",
                "commitment_type",
                "commitment_windowed",
                "effective_from",
                "metadata",
                "tier_mode",
                "tiers",
                "transform_quantity",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .filtercondition import FilterCondition, FilterConditionTypedDict
from .sortcondition import SortCondition, SortConditionTypedDict
from .status import Status
from .transactionreason import TransactionReason
from .transactionstatus import TransactionStatus
from .transactiontype import TransactionType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


WalletTransactionFilterOrder = Literal[
    "asc",
    "desc",
]


class WalletTransactionFilterTypedDict(TypedDict):
    created_by: NotRequired[str]
    credits_available_gt: NotRequired[float]
    end_time: NotRequired[str]
    expand: NotRequired[str]
    expiry_date_after: NotRequired[str]
    expiry_date_before: NotRequired[str]
    filters: NotRequired[List[FilterConditionTypedDict]]
    r"""filters allows complex filtering based on multiple fields"""
    id: NotRequired[str]
    limit: NotRequired[int]
    offset: NotRequired[int]
    order: NotRequired[WalletTransactionFilterOrder]
    priority: NotRequired[int]
    reference_id: NotRequired[str]
    reference_type: NotRequired[str]
    sort: NotRequired[List[SortConditionTypedDict]]
    start_time: NotRequired[str]
    status: NotRequired[Status]
    transaction_reason: NotRequired[TransactionReason]
    transaction_status: NotRequired[TransactionStatus]
    type: NotRequired[TransactionType]


class WalletTransactionFilter(BaseModel):
    created_by: Optional[str] = None

    credits_available_gt: Optional[float] = None

    end_time: Optional[str] = None

    expand: Optional[str] = None

    expiry_date_after: Optional[str] = None

    expiry_date_before: Optional[str] = None

    filters: Optional[List[FilterCondition]] = None
    r"""filters allows complex filtering based on multiple fields"""

    id: Optional[str] = None

    limit: Optional[int] = None

    offset: Optional[int] = None

    order: Optional[WalletTransactionFilterOrder] = None

    priority: Optional[int] = None

    reference_id: Optional[str] = None

    reference_type: Optional[str] = None

    sort: Optional[List[SortCondition]] = None

    start_time: Optional[str] = None

    status: Optional[Status] = None

    transaction_reason: Optional[TransactionReason] = None

    transaction_status: Optional[TransactionStatus] = None

    type: Optional[TransactionType] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "created_by",
                "credits_available_gt",
                "end_time",
                "expand",
                "expiry_date_after",
                "expiry_date_before",
                "filters",
                "id",
                "limit",
                "offset",
                "order",
                "priority",
                "reference_id",
                "reference_type",
                "sort",
                "start_time",
                "status",
                "transaction_reason",
                "transaction_status",
                "type",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_customerresponse import DtoCustomerResponse, DtoCustomerResponseTypedDict
from .dto_userresponse import DtoUserResponse, DtoUserResponseTypedDict
from .dto_walletresponse import DtoWalletResponse, DtoWalletResponseTypedDict
from .status import Status
from .transactionreason import TransactionReason
from .transactionstatus import TransactionStatus
from .transactiontype import TransactionType
from .wallettxreferencetype import WalletTxReferenceType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoWalletTransactionResponseTypedDict(TypedDict):
    amount: NotRequired[str]
    conversion_rate: NotRequired[str]
    r"""conversion_rate is the conversion rate for the transaction to the currency"""
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    created_by_user: NotRequired[DtoUserResponseTypedDict]
    credit_amount: NotRequired[str]
    credit_balance_after: NotRequired[str]
    credit_balance_before: NotRequired[str]
    credits_available: NotRequired[str]
    currency: NotRequired[str]
    customer: NotRequired[DtoCustomerResponseTypedDict]
    r"""Customer response object containing all customer information"""
    customer_id: NotRequired[str]
    description: NotRequired[str]
    environment_id: NotRequired[str]
    expiry_date: NotRequired[str]
    id: NotRequired[str]
    idempotency_key: NotRequired[str]
    metadata: NotRequired[Dict[str, str]]
    priority: NotRequired[int]
    reference_id: NotRequired[str]
    reference_type: NotRequired[WalletTxReferenceType]
    status: NotRequired[Status]
    tenant_id: NotRequired[str]
    topup_conversion_rate: NotRequired[str]
    r"""topup_conversion_rate is the conversion rate for the topup to the currency"""
    transaction_reason: NotRequired[TransactionReason]
    transaction_status: NotRequired[TransactionStatus]
    type: NotRequired[TransactionType]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]
    wallet: NotRequired[DtoWalletResponseTypedDict]
    wallet_id: NotRequired[str]


class DtoWalletTransactionResponse(BaseModel):
    amount: Optional[str] = None

    conversion_rate: Optional[str] = None
    r"""conversion_rate is the conversion rate for the transaction to the currency"""

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    created_by_user: Optional[DtoUserResponse] = None

    credit_amount: Optional[str] = None

    credit_balance_after: Optional[str] = None

    credit_balance_before: Optional[str] = None

    credits_available: Optional[str] = None

    currency: Optional[str] = None

    customer: Optional[DtoCustomerResponse] = None
    r"""Customer response object containing all customer information"""

    customer_id: Optional[str] = None

    description: Optional[str] = None

    environment_id: Optional[str] = None

    expiry_date: Optional[str] = None

    id: Optional[str] = None

    idempotency_key: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    priority: Optional[int] = None

    reference_id: Optional[str] = None

    reference_type: Optional[WalletTxReferenceType] = None

    status: Optional[Status] = None

    tenant_id: Optional[str] = None

    topup_conversion_rate: Optional[str] = None
    r"""topup_conversion_rate is the conversion rate for the topup to the currency"""

    transaction_reason: Optional[TransactionReason] = None

    transaction_status: Optional[TransactionStatus] = None

    type: Optional[TransactionType] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    wallet: Optional[DtoWalletResponse] = None

    wallet_id: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount",
                "conversion_rate",
                "created_at",
                "created_by",
                "created_by_user",
                "credit_amount",
                "credit_balance_after",
                "credit_balance_before",
                "credits_available",
                "currency",
                "customer",
                "customer_id",
                "description",
                "environment_id",
                "expiry_date",
                "id",
                "idempotency_key",
                "metadata",
                "priority",
                "reference_id",
                "reference_type",
                "status",
                "tenant_id",
                "topup_conversion_rate",
                "transaction_reason",
                "transaction_status",
                "type",
                "updated_at",
                "updated_by",
                "wallet",
                "wallet_id",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import UnrecognizedStr
from typing import Literal, Union


SecretProvider = Union[
    Literal[
        "flexprice",
        "stripe",
        "s3",
        "hubspot",
        "razorpay",
        "chargebee",
        "quickbooks",
        "nomod",
        "moyasar",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_costpoint import DtoCostPoint, DtoCostPointTypedDict
from .meter_meter import MeterMeter, MeterMeterTypedDict
from .price_price import PricePrice, PricePriceTypedDict
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCostAnalyticItemTypedDict(TypedDict):
    cost_by_period: NotRequired[List[DtoCostPointTypedDict]]
    r"""Breakdown"""
    costsheet_id: NotRequired[str]
    currency: NotRequired[str]
    r"""Metadata"""
    customer_id: NotRequired[str]
    external_customer_id: NotRequired[str]
    meter: NotRequired[MeterMeterTypedDict]
    meter_id: NotRequired[str]
    meter_name: NotRequired[str]
    price: NotRequired[PricePriceTypedDict]
    price_id: NotRequired[str]
    properties: NotRequired[Dict[str, str]]
    source: NotRequired[str]
    total_cost: NotRequired[str]
    r"""Aggregated metrics"""
    total_events: NotRequired[int]
    total_quantity: NotRequired[str]


class DtoCostAnalyticItem(BaseModel):
    cost_by_period: Optional[List[DtoCostPoint]] = None
    r"""Breakdown"""

    costsheet_id: Optional[str] = None

    currency: Optional[str] = None
    r"""Metadata"""

    customer_id: Optional[str] = None

    external_customer_id: Optional[str] = None

    meter: Optional[MeterMeter] = None

    meter_id: Optional[str] = None

    meter_name: Optional[str] = None

    price: Optional[PricePrice] = None

    price_id: Optional[str] = None

    properties: Optional[Dict[str, str]] = None

    source: Optional[str] = None

    total_cost: Optional[str] = None
    r"""Aggregated metrics"""

    total_events: Optional[int] = None

    total_quantity: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "cost_by_period",
                "costsheet_id",
                "currency",
                "customer_id",
                "external_customer_id",
                "meter",
                "meter_id",
                "meter_name",
                "price",
                "price_id",
                "properties",
                "source",
                "total_cost",
                "total_events",
                "total_quantity",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_taxrateresponse import DtoTaxRateResponse, DtoTaxRateResponseTypedDict
from .status import Status
from .taxrateentitytype import TaxRateEntityType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoTaxAppliedResponseTypedDict(TypedDict):
    applied_at: NotRequired[str]
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    currency: NotRequired[str]
    entity_id: NotRequired[str]
    entity_type: NotRequired[TaxRateEntityType]
    environment_id: NotRequired[str]
    id: NotRequired[str]
    idempotency_key: NotRequired[str]
    metadata: NotRequired[Dict[str, str]]
    status: NotRequired[Status]
    tax_amount: NotRequired[str]
    tax_association_id: NotRequired[str]
    tax_rate: NotRequired[DtoTaxRateResponseTypedDict]
    tax_rate_id: NotRequired[str]
    taxable_amount: NotRequired[str]
    tenant_id: NotRequired[str]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]


class DtoTaxAppliedResponse(BaseModel):
    applied_at: Optional[str] = None

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    currency: Optional[str] = None

    entity_id: Optional[str] = None

    entity_type: Optional[TaxRateEntityType] = None

    environment_id: Optional[str] = None

    id: Optional[str] = None

    idempotency_key: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    status: Optional[Status] = None

    tax_amount: Optional[str] = None

    tax_association_id: Optional[str] = None

    tax_rate: Optional[DtoTaxRateResponse] = None

    tax_rate_id: Optional[str] = None

    taxable_amount: Optional[str] = None

    tenant_id: Optional[str] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "applied_at",
                "created_at",
                "created_by",
                "currency",
                "entity_id",
                "entity_type",
                "environment_id",
                "id",
                "idempotency_key",
                "metadata",
                "status",
                "tax_amount",
                "tax_association_id",
                "tax_rate",
                "tax_rate_id",
                "taxable_amount",
                "tenant_id",
                "updated_at",
                "updated_by",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_debugtracker import DtoDebugTracker, DtoDebugTrackerTypedDict
from .dto_event import DtoEvent, DtoEventTypedDict
from .dto_featureusageinfo import DtoFeatureUsageInfo, DtoFeatureUsageInfoTypedDict
from .eventprocessingstatustype import EventProcessingStatusType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoGetEventByIDResponseTypedDict(TypedDict):
    debug_tracker: NotRequired[DtoDebugTrackerTypedDict]
    event: NotRequired[DtoEventTypedDict]
    processed_events: NotRequired[List[DtoFeatureUsageInfoTypedDict]]
    status: NotRequired[EventProcessingStatusType]


class DtoGetEventByIDResponse(BaseModel):
    debug_tracker: Optional[DtoDebugTracker] = None

    event: Optional[DtoEvent] = None

    processed_events: Optional[List[DtoFeatureUsageInfo]] = None

    status: Optional[EventProcessingStatusType] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["debug_tracker", "event", "processed_events", "status"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.models.components import (
    dto_cancelschedulerequest as components_dto_cancelschedulerequest,
)
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from flexprice_py.utils import FieldMetadata, PathParamMetadata, RequestMetadata
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CancelSubscriptionScheduleRequestTypedDict(TypedDict):
    schedule_id_param: str
    r"""Schedule ID (optional if using request body)"""
    body: NotRequired[
        components_dto_cancelschedulerequest.DtoCancelScheduleRequestTypedDict
    ]
    r"""Cancel request (optional if using path parameter)"""


class CancelSubscriptionScheduleRequest(BaseModel):
    schedule_id_param: Annotated[
        str,
        pydantic.Field(alias="schedule_id"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""Schedule ID (optional if using request body)"""

    body: Annotated[
        Optional[components_dto_cancelschedulerequest.DtoCancelScheduleRequest],
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ] = None
    r"""Cancel request (optional if using path parameter)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["body"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_event import DtoEvent, DtoEventTypedDict
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoGetEventsResponseTypedDict(TypedDict):
    events: NotRequired[List[DtoEventTypedDict]]
    has_more: NotRequired[bool]
    iter_first_key: NotRequired[str]
    iter_last_key: NotRequired[str]
    offset: NotRequired[int]
    total_count: NotRequired[int]


class DtoGetEventsResponse(BaseModel):
    events: Optional[List[DtoEvent]] = None

    has_more: Optional[bool] = None

    iter_first_key: Optional[str] = None

    iter_last_key: Optional[str] = None

    offset: Optional[int] = None

    total_count: Optional[int] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "events",
                "has_more",
                "iter_first_key",
                "iter_last_key",
                "offset",
                "total_count",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_taxrateresponse import DtoTaxRateResponse, DtoTaxRateResponseTypedDict
from .status import Status
from .taxrateentitytype import TaxRateEntityType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoTaxAssociationResponseTypedDict(TypedDict):
    auto_apply: NotRequired[bool]
    r"""Whether this tax should be automatically applied"""
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    currency: NotRequired[str]
    r"""Currency"""
    entity_id: NotRequired[str]
    r"""ID of the entity this tax rate applies to"""
    entity_type: NotRequired[TaxRateEntityType]
    environment_id: NotRequired[str]
    r"""EnvironmentID is the ID of the environment this tax rate config belongs to"""
    id: NotRequired[str]
    r"""ID of the ent."""
    metadata: NotRequired[Dict[str, str]]
    r"""Metadata holds the value of the \"metadata\" field."""
    priority: NotRequired[int]
    r"""Priority for tax resolution (lower number = higher priority)"""
    status: NotRequired[Status]
    tax_rate: NotRequired[DtoTaxRateResponseTypedDict]
    tax_rate_id: NotRequired[str]
    r"""Reference to the TaxRate entity"""
    tenant_id: NotRequired[str]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]


class DtoTaxAssociationResponse(BaseModel):
    auto_apply: Optional[bool] = None
    r"""Whether this tax should be automatically applied"""

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    currency: Optional[str] = None
    r"""Currency"""

    entity_id: Optional[str] = None
    r"""ID of the entity this tax rate applies to"""

    entity_type: Optional[TaxRateEntityType] = None

    environment_id: Optional[str] = None
    r"""EnvironmentID is the ID of the environment this tax rate config belongs to"""

    id: Optional[str] = None
    r"""ID of the ent."""

    metadata: Optional[Dict[str, str]] = None
    r"""Metadata holds the value of the \"metadata\" field."""

    priority: Optional[int] = None
    r"""Priority for tax resolution (lower number = higher priority)"""

    status: Optional[Status] = None

    tax_rate: Optional[DtoTaxRateResponse] = None

    tax_rate_id: Optional[str] = None
    r"""Reference to the TaxRate entity"""

    tenant_id: Optional[str] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "auto_apply",
                "created_at",
                "created_by",
                "currency",
                "entity_id",
                "entity_type",
                "environment_id",
                "id",
                "metadata",
                "priority",
                "status",
                "tax_rate",
                "tax_rate_id",
                "tenant_id",
                "updated_at",
                "updated_by",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class MeterFilterTypedDict(TypedDict):
    key: NotRequired[str]
    r"""Key is the key for the filter from $event.properties
    Currently we support only first level keys in the properties and not nested keys
    """
    values: NotRequired[List[str]]
    r"""Values are the possible values for the filter to be considered for the meter
    For ex \"model_name\" could have values \"o1-mini\", \"gpt-4o\" etc
    """


class MeterFilter(BaseModel):
    key: Optional[str] = None
    r"""Key is the key for the filter from $event.properties
    Currently we support only first level keys in the properties and not nested keys
    """

    values: Optional[List[str]] = None
    r"""Values are the possible values for the filter to be considered for the meter
    For ex \"model_name\" could have values \"o1-mini\", \"gpt-4o\" etc
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["key", "values"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingperiod import BillingPeriod
from .commitmenttype import CommitmentType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CommitmentInfoTypedDict(TypedDict):
    amount: NotRequired[str]
    computed_commitment_utilized_amount: NotRequired[str]
    computed_overage_amount: NotRequired[str]
    computed_true_up_amount: NotRequired[str]
    r"""total_cost = computed_commitment_utilized_amount + computed_overage_amount + computed_true_up_amount"""
    duration: NotRequired[BillingPeriod]
    is_windowed: NotRequired[bool]
    overage_factor: NotRequired[str]
    quantity: NotRequired[str]
    r"""Only used for quantity-based commitments"""
    true_up_enabled: NotRequired[bool]
    type: NotRequired[CommitmentType]


class CommitmentInfo(BaseModel):
    amount: Optional[str] = None

    computed_commitment_utilized_amount: Optional[str] = None

    computed_overage_amount: Optional[str] = None

    computed_true_up_amount: Optional[str] = None
    r"""total_cost = computed_commitment_utilized_amount + computed_overage_amount + computed_true_up_amount"""

    duration: Optional[BillingPeriod] = None

    is_windowed: Optional[bool] = None

    overage_factor: Optional[str] = None

    quantity: Optional[str] = None
    r"""Only used for quantity-based commitments"""

    true_up_enabled: Optional[bool] = None

    type: Optional[CommitmentType] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount",
                "computed_commitment_utilized_amount",
                "computed_overage_amount",
                "computed_true_up_amount",
                "duration",
                "is_windowed",
                "overage_factor",
                "quantity",
                "true_up_enabled",
                "type",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import UnrecognizedStr
from typing import Literal, Union


AggregationType = Union[
    Literal[
        "COUNT",
        "SUM",
        "AVG",
        "COUNT_UNIQUE",
        "LATEST",
        "SUM_WITH_MULTIPLIER",
        "MAX",
        "WEIGHTED_SUM",
    ],
    UnrecognizedStr,
]
